!{\src2tex{textfont=tt}}
!!****f* ABINIT/cppm3par
!! NAME
!! cppm3par
!!
!! FUNCTION
!! Calculate the plasmon-pole parameters using Linden-Horsh (PRB 37, 8351, 1988)  
!!
!! COPYRIGHT
!! Copyright (C) 1999-2006 ABINIT group (RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  epsm1(npwvec,npwvec,nomega,nq)=dielectric matrix at nomega frequencies, and nq wavevectors
!!  npwvec=number of plane waves
!!  nomega=number of frequencies (usually 2)
!!  nq=number of q points
!!  omega(nomega)=frequencies
!!  omegaplasma=plasma frequency
!!  qratio=(q+G).(q+G')/(|q+G|.|q+G"|)
!!  rho=charge deinsity on real space FFT grid
!!  ngfft1,ngfft2,ngfft3=FFT grid
!!  
!! OUTPUT
!!  bigomegatwsq(npwvec,npwvec,nq)=parameter of the plasmon-pole model (see gwa.pdf file)
!!  omegatw(npwvec,npwvec,nq)=parameter of the plasmon-pole model (see gwa.pdf file)
!!  eigtot=the eigvec of the dielctric matrix
!!
!! PARENTS
!!      sigma
!!
!! CHILDREN
!!      cggfft,chpev,fourdp,leave_new,wrtout,zhpev
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

 subroutine cppm3par(npwvec,nq,nomega,epsm1,omega,bigomegatwsq,omegatw,&
& omegaplasma,ngfft1,ngfft2,ngfft3,gvec,qratio,rho,q,nr,b1,b2,b3,eigtot,mpi_enreg)

 use defs_basis
 use defs_datatypes 

 implicit none
 
!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ngfft1,ngfft2,ngfft3,nomega,npwvec,nq,nr
 real(dp),intent(in) :: omegaplasma
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 integer,intent(in) :: gvec(3,npwvec)
 real(dp),intent(in) :: b1(3),b2(3),b3(3),q(nq),qratio(npwvec,npwvec,nq)
 real(dp),intent(in) :: rho(nr)
 complex,intent(in) :: epsm1(npwvec,npwvec,nomega,nq),omega(nomega)
 complex,intent(out) :: bigomegatwsq(npwvec,npwvec,nq),eigtot(npwvec,npwvec,nq)
 complex,intent(out) :: omegatw(npwvec,npwvec,nq)
 
!Local variables-------------------------------
!scalars
 integer :: gmg01,gmg02,gmg03,ierr,ig,igp,ii,index,iq,istat,jj,kk,modx
 integer :: tim_fourdp=2
 real(dp) :: omegaplasmasq
 complex :: rho0
 character(len=1) :: jobs,uplo
 character(len=5000) :: message
!arrays
 integer :: gdiff(3),ngfft(18)
 integer,allocatable :: igfft(:,:)
 real(dp),allocatable :: eigval(:),qplusg(:),rhog_dp(:,:),zhpev2(:)
!no_abirules
 complex::conjg_eig
 COMPLEX*16,allocatable::matr(:),zhpev1(:),eigvec(:,:)
 COMPLEX*16,allocatable::omegat(:,:),zz(:,:),temp(:,:),tmp_eig(:)
 COMPLEX,allocatable :: omegatwsq(:,:),rhog(:),rhogg(:,:)

#ifdef VMS
!DEC$ ATTRIBUTES ALIAS:'ZHPEV' :: zhpev
#endif

!*************************************************************************
! calculmat the density in G space rhog(G)
  allocate(rhog_dp(2,nr),stat=istat)
  if(istat/=0) stop 'rhog_dp out of memory'
  allocate(rhog(nr),stat=istat)
  if(istat/=0) stop 'rhog out of memory'
  allocate(igfft(npwvec,npwvec),stat=istat)
  if(istat/=0) stop 'igfft out of memory'
  allocate(rhogg(npwvec,npwvec),stat=istat)
  if(istat/=0) stop 'rhogg out of memory'


  ngfft(1)=ngfft1
  ngfft(2)=ngfft2
  ngfft(3)=ngfft3
  ngfft(4)=2*(ngfft(1)/2)+1
  ngfft(5)=2*(ngfft(2)/2)+1
  ngfft(6)=ngfft(3)
  ngfft(7)=100
  ngfft(8)=256
  ngfft(9)=0
  ngfft(10)=1
  ngfft(11)=0
  ngfft(12)=ngfft2
  ngfft(13)=ngfft3
  ngfft(14)=0
 
! conduct FFT to rhog

 call fourdp(1,rhog_dp,rho,-1,mpi_enreg,nr,ngfft,0)
 rhog(1:nr)=cmplx(rhog_dp(1,1:nr),rhog_dp(2,1:nr))

! calculate the FFT index of each (G-G') vector and assign the value
! of correspondent density simultanously

 call cggfft(npwvec,ngfft1,ngfft2,ngfft3,gvec,igfft)

 do ig=1,npwvec
  do igp=1,npwvec
   rhogg(ig,igp)=rhog(igfft(ig,igp))
  end do
 end do

! rho0=rhog(igfft(1,1))
! omegaplasmasq=omegaplasma*omegaplasma
! rhogg(:,:)=rhogg(:,:)/rho0*omegaplasmasq

  rhogg(:,:)=rhogg(:,:)*4*pi


!Now we have rhogg,rho0

 deallocate(rhog_dp,rhog,igfft)

! calculate the dielectric matrix eignvalues and vectors


! only we will use the static DM i.e., only the w=0 part (eps(:,:,1,:))


  allocate(eigval(npwvec),stat=istat)
  if(istat/=0) stop 'eigval out of memory'
  allocate(eigvec(npwvec,npwvec),stat=istat)  ! eigenvalues and vectors of DM
  if(istat/=0) stop 'eigvec out of memory'
  allocate(temp(npwvec,npwvec),stat=istat) 
  if(istat/=0) stop 'temp out of memory'
  allocate(omegat(npwvec,npwvec),stat=istat)
  if(istat/=0) stop 'omegat of memory'
  allocate(qplusg(npwvec),stat=istat)
  if(istat/=0) stop 'qplusg of memory'
  allocate(zz(npwvec,nq),stat=istat)
  if(istat/=0) stop 'zz of memory'

 
 zz(:,:)=0.0

 do iq=1,nq


! Store the susceptibility matrix in upper mode before calling zhpev for each iq value

 allocate(matr(npwvec*(npwvec+1)/2),stat=istat)
 if(istat/=0) stop 'matr of memory'

  index=1
  do ii=1,npwvec
   do jj=1,ii
    matr(index)=epsm1(jj,ii,1,iq)
    index=index+1
   end do
  end do

 allocate(zhpev2(3*npwvec-2),stat=istat)
 if(istat/=0) stop 'qplusg of memory'
 allocate(zhpev1(2*npwvec-1),stat=istat)
 if(istat/=0) stop 'qplusg of memory' ! woking arrays for lapack



#if defined T3E
  call CHPEV('V','U',npwvec,matr,eigval,eigvec,npwvec,&
& zhpev1,zhpev2,ierr)
#else
  call zhpev('V','U',npwvec,matr,eigval,eigvec,npwvec,&
 &zhpev1,zhpev2,ierr)
#endif


 deallocate(matr,zhpev2,zhpev1) ! woking arrays for lapack


 if (ierr<0) then
      write (message,'(a,a,i4,a,a)') &
&    'failed to calculate the eignvalues and eignvaectors of the dielectric matrix ',ch10,&
&     ierr*(-1),'th argument in the matrix had an illegal value',ch10
      call wrtout(ab_out,message,'COLL')
      call wrtout(std_out,message,'COLL')
      call leave_new('COLL')
     end if

 if (ierr>0) then
      write (message,'(a,a,a,i4,a,a,a)') &
&    'failed to calculate the eignvalues and eignvaectors of the dielectric matrix ',ch10,&
&    'the algorithm failed to converge;', ierr, 'off-diagonal elements of an intermediate tridiagonal form',ch10,&
&    'did not converge to zero.',ch10
      call wrtout(ab_out,message,'COLL')
      call wrtout(std_out,message,'COLL')
      call leave_new('COLL')
     end if

    temp(:,:)=qratio(:,:,iq)*rhogg(:,:)
    
! calculate the PPM parameters and the eignpotentials needed for the calculation
! of the generalized overlap matrix
! Note: the eignpotentials has to be calculated on the FFT (G-Gpr) index

  eigtot(:,:,iq)=eigvec(:,:)


  do ii=1,npwvec
   do ig=1,npwvec
      conjg_eig=conjg(eigvec(ig,ii))
     do igp=1,npwvec
       zz(ii,iq)=zz(ii,iq)+conjg_eig*temp(ig,igp)*eigvec(igp,ii)
     end do
  end do
      omegat(ii,iq)=sqrt(real(zz(ii,iq))/(1-eigval(ii)))
   end do
      
          


 end do !iq
 deallocate(rhogg)

! normally the PPM parameters in this model depond only on the DM band index and the q point ( 1 dimentional array at each q point)
! for technical reasons related to the way we will proceed into the calculation of 
! self energy in the sigme.F90, we will copy the values of the ppm into ( the diagonal of 2 dimentional array at each q point)


  do ii=1,npwvec
   bigomegatwsq(ii,ii,:)=zz(ii,:)
   omegatw(ii,ii,:)=omegat(ii,:)
  end do

  deallocate(temp)
  deallocate(eigval,omegat,zz)
  deallocate(eigvec)
  deallocate(qplusg)
 end subroutine cppm3par
!!***
