!{\src2tex{textfont=tt}}
!!****f* ABINIT/cppm4par
!! NAME
!! cppm4par
!!
!! FUNCTION
!! Calculate the plasmon-pole parameters using Engel and Farid model (PRB47,15931,1993)
!!
!! COPYRIGHT
!! Copyright (C) 1999-2006 ABINIT group (RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  epsm1(npwvec,npwvec,nomega,nq)=dielectric matrix at nomega frequencies, and nq wavevectors
!!  mpi_enreg=informations about MPI parallelization
!!  npwvec=number of plane waves
!!  nomega=number of frequencies (usually 2)
!!  nq=number of q points
!!  omega(nomega)=frequencies
!!  omegaplasma=input variable
!!  rho=charge deinsity on real space FFT grid
!!  
!! OUTPUT
!!  bigomegatwsq(npwvec,npwvec,nq)=parameter of the plasmon-pole model (see gwa.pdf file)
!!  omegatw(npwvec,npwvec,nq)=parameter of the plasmon-pole model (see gwa.pdf file)
!!
!! PARENTS
!!      sigma
!!
!! CHILDREN
!!      fourdp
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

 subroutine cppm4par(npwvec,nq,epsm1,nomega,bigomegatwsq,omegatw,&
& ngfft1,ngfft2,ngfft3,gvec,rho,nr,q,b1,b2,b3,mpi_enreg)

 use defs_basis
 use defs_datatypes
 
 implicit none
 
!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ngfft1,ngfft2,ngfft3,nomega,npwvec,nq,nr
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 integer,intent(in) :: gvec(3,npwvec)
 real(dp),intent(in) :: b1(3),b2(3),b3(3),q(3,nq),rho(nr)
 complex,intent(in) :: epsm1(npwvec,npwvec,nomega,nq)
 complex,intent(out) :: bigomegatwsq(npwvec,npwvec,nq)
 complex,intent(out) :: omegatw(npwvec,npwvec,nq)
 
!Local variables-------------------------------
!scalars
 integer :: gmg01,gmg02,gmg03,ierr,ig,igp,ii,iii,index,iq,istat,jj,lowork,modx
 integer :: tim_fourdp=2
 real(dp) :: delta,qpg_dot_qpgp,rhoav
 complex :: rho0
 character(len=1) :: jobs,uplo
!arrays
 integer :: gdiff(3),ngfft(18)
 integer,allocatable :: igfft(:,:)
 real(dp) :: gppq(3),gpq(3)
 real(dp),allocatable :: eigval(:),qplusg(:),rhog_dp(:,:),rwork(:),zhpev2(:)
 complex,allocatable :: chi0(:,:,:),chitmp(:,:),omegatwsq(:,:),rhog(:)
 complex,allocatable :: rhogg(:,:),zz1(:,:),zz2(:,:)
!no_abirules
 COMPLEX*16,allocatable::mtemp(:,:),work(:),eigvec(:,:),mm(:,:,:),chitmps(:,:),tmp1(:)
 COMPLEX*16,allocatable::zhpev1(:),matr(:)
 
!*************************************************************************

#ifdef VMS
!DEC$ ATTRIBUTES ALIAS:'ZHEGV' :: zhegv
#endif

! calculat the density in G space rhog(G)


 allocate(rhog_dp(2,nr),stat=istat)
 if(istat/=0) stop 'rhog_dp out of memory'
 allocate(rhog(nr),stat=istat)
 if(istat/=0) stop 'rhog out of memory'
 allocate(igfft(npwvec,npwvec),stat=istat)
 if(istat/=0) stop 'igfft out of memory'
 allocate(rhogg(npwvec,npwvec),stat=istat)
 if(istat/=0) stop 'rhogg out of memory'

  ngfft(1)=ngfft1
  ngfft(2)=ngfft2
  ngfft(3)=ngfft3
  ngfft(4)=2*(ngfft(1)/2)+1
  ngfft(5)=2*(ngfft(2)/2)+1
  ngfft(6)=ngfft(3)
  ngfft(7)=100
  ngfft(8)=256
  ngfft(9)=0
  ngfft(10)=1
  ngfft(11)=0
  ngfft(12)=ngfft2
  ngfft(13)=ngfft3
  ngfft(14)=0
 
! conduct FFT to rhog

 call fourdp(1,rhog_dp,rho,-1,mpi_enreg,nr,ngfft,0)
 rhog(1:nr)=cmplx(rhog_dp(1,1:nr),rhog_dp(2,1:nr))

! calculate the FFT index of each (G-G') vector and assign the value
! of correspondent density simultanously

 deallocate(rhog_dp)

 call cggfft(npwvec,ngfft1,ngfft2,ngfft3,gvec,igfft)

 do ig=1,npwvec
  do igp=1,npwvec
   rhogg(ig,igp)=rhog(igfft(ig,igp))
  end do
 end do


 deallocate(igfft)
 deallocate(rhog)


!Now we have rhogg,rho0


! calculate the m matrix 

 allocate(mm(npwvec,npwvec,nq),stat=istat)
 if(istat/=0) stop 'mtemp out of memory'
 
 
 do iq=1,nq
   do ig=1,npwvec 
    gpq(1)=gvec(1,ig)+q(1,iq)
    gpq(2)=gvec(2,ig)+q(2,iq)
    gpq(3)=gvec(3,ig)+q(3,iq)
   do igp=1,npwvec
    gppq(1)=gvec(1,igp)+q(1,iq)
    gppq(2)=gvec(2,igp)+q(2,iq)
    gppq(3)=gvec(3,igp)+q(3,iq)
    qpg_dot_qpgp=0
    do ii=1,3
     qpg_dot_qpgp=qpg_dot_qpgp+&
&     (gpq(1)*b1(ii) +gpq(2)*b2(ii) +gpq(3)*b3(ii))*&
&     (gppq(1)*b1(ii)+gppq(2)*b2(ii)+gppq(3)*b3(ii))
    end do
    mm(ig,igp,iq)= rhogg(ig,igp)*qpg_dot_qpgp

   end do     !igp
  end do    !ig
 end do  !iq

 allocate(chitmp(npwvec,npwvec),stat=istat)
 if(istat/=0) stop 'chitmp out of memory'
 allocate(chi0(npwvec,npwvec,iq),stat=istat)
 if(istat/=0) stop 'chi0 out of memory'
 allocate(qplusg(npwvec),stat=istat)
 if(istat/=0) stop 'qplusg out of memory'

! extract the linear polarizability from eps1
! eps1=1+chitmp*Vc

 do iq=1,nq
  chitmp(:,:)=epsm1(:,:,1,iq)
  call cvc(nq,iq,q,b1,b2,b3,npwvec,gvec,qplusg)
  do ig=1,npwvec
   chitmp(ig,ig)=chitmp(ig,ig)-1
  end do
   do ig=1,npwvec
    do igp=1,npwvec
     chi0(ig,igp,iq)=chitmp(ig,igp)*qplusg(ig)*qplusg(igp)/(4.0*pi)
    end do
   end do 
end do

 deallocate(chitmp)
 

! solve chi0*Y=Lambda*M*Y



do iq=1,nq

 allocate(eigval(npwvec),stat=istat)
 if(istat/=0) stop 'eigval out of memory'
 allocate(eigvec(npwvec,npwvec),stat=istat)
 if(istat/=0) stop 'eigvec out of memory'
 allocate(mtemp(npwvec,npwvec),stat=istat)
 if(istat/=0) stop 'mtemp out of memory'
 allocate(chitmps(npwvec,npwvec),stat=istat)       ! temp working arrays
 if(istat/=0) stop 'chitmps out of memory'
 allocate(work(2*npwvec-1),stat=istat)
 if(istat/=0) stop 'work out of memory'
 allocate(rwork(3*npwvec-2),stat=istat) ! needed by lapack
 if(istat/=0) stop 'rwork out of memory'



!! dubbeging perpose
! allocate(matr(npwvec*(npwvec+1)/2))
! if(istat/=0) stop 'matr out of memory'
! allocate(zhpev2(2*npwvec-1),stat=istat)
! if(istat/=0) stop 'work out of memory'
! allocate(zhpev1(3*npwvec-2),stat=istat) ! needed by lapack
! if(istat/=0) stop 'rwork out of memory'

!  index=1
!  do ii=1,npwvec
!   do jj=1,ii
!    matr(index)=chi0(jj,ii,iq)
!    index=index+1
!   end do
!  end do


!  call zhpev('v','u',npwvec,matr,eigval,eigvec,npwvec,&
! &zhpev1,zhpev2,ierr)


! deallocate(zhpev1,zhpev2,matr)

!! dubbeging perpose


! copy chi0, m into working arrays
  chitmps(:,:)=chi0(:,:,iq)
  mtemp(:,:)=mm(:,:,iq)
  lowork=2*npwvec-1
  call zhegv(1,'v','u',npwvec,chitmps,npwvec,mtemp,npwvec,eigval,work,lowork,rwork,ierr)

  eigvec(:,:)=chitmps(:,:)


 deallocate(mtemp)
 deallocate(chitmps)       ! temp working arrays
 deallocate(work)
 deallocate(rwork) ! needed by lapack
 
! start the plasmon pole parameters
 allocate(tmp1(npwvec),stat=istat)   ! eign vectors
 if(istat/=0) stop 'tmp1 out of memory'
 allocate(zz1(npwvec,npwvec),stat=istat)                ! checking
 if(istat/=0) stop 'zz out of memory'
 allocate(zz2(npwvec,npwvec),stat=istat)                ! checking
 if(istat/=0) stop 'zz out of memory'


! good note:
! please note that the lowest plasmon energy on gamma should be
! close to the plsma energy

  zz2(:,:)=0.0d0
  call cvc(nq,iq,q,b1,b2,b3,npwvec,gvec,qplusg)
  do ii=1,npwvec
   omegatw(ii,ii,iq)=sqrt(-1/eigval(ii))
   tmp1(:)=eigvec(:,ii)
    do ig=1,npwvec
     do igp=1,npwvec
      if(qplusg(ig)<0.001)then
       if(qplusg(igp)<0.001)then
!       q=0=G=G"
        zz1(ig,igp)=0.0
       else
!       q=0=G;G"/=0
        zz1(ig,igp)=rhogg(ig,igp)
       end if
      else
       zz1(ig,igp)=mm(ig,igp,iq)/(qplusg(ig))
       if(qplusg(igp)<0.001)then
!      q=0=G";G/=0
        zz1(ig,igp)=0.0
       end if
      end if
!      zz2(ig,ii)=zz2(ig,ii)+zz1(ig,igp)*tmp1(igp)
       zz2(ig,ii)=zz2(ig,ii)+zz1(ig,igp)*eigvec(igp,ii)
     end do
    bigomegatwsq(ig,ii,iq)=sqrt(4.0*pi)*zz2(ig,ii)
    end do   !ig
  end do   !ii

 deallocate(tmp1)   ! eign vectors
 deallocate(eigvec)
 deallocate(eigval)
 deallocate(zz1)                ! checking
 deallocate(zz2)
 
 end do ! iq

  deallocate(qplusg)
  deallocate(chi0)
  deallocate(rhogg)
  deallocate(mm)
do iq=1,nq
   write(1,'(2x,i3,3x,10f9.5)')iq,(real(27.2116*omegatw(ig,ig,iq)),ig=1,10)
end do

 
 end subroutine cppm4par
!!***
